//
//  TATUserDataHelper.m
//  TATMediaSDK
//
//  Created by wuleslie on 2019/12/4.
//  Copyright © 2019 wuleslie. All rights reserved.
//

#import "TATUserDataHelper.h"
#import "NSDate+TATFormatter.h"
#import "TATClient.h"
#import "NSString+TATBase.h"
#import "NSObject+TATModel.h"
#import "NSData+TATGZip.h"
#import "TATBase64.h"

@implementation TATUserDataHelper

+ (NSString *)amassMdStringWithBlockList:(NSArray *)blockList {
    NSDictionary *params = [TATUserDataHelper subSetOfParams:[TATUserDataHelper mdParamsForCollectAPI] byTrimmingList:blockList];
    return [TATUserDataHelper handleMDParams:params];
}

+ (NSDictionary *)subSetOfParams:(NSDictionary *)params byTrimmingList:(NSArray *)blockList {
    if (!blockList) {
        return params;
    }
    NSMutableDictionary *dictionary = [NSMutableDictionary dictionaryWithDictionary:params];
    for (NSString *key in blockList) {
        [dictionary removeObjectForKey:key];
    }
    return dictionary.copy;
}

+ (NSString *)sevingMDString {
    return [TATUserDataHelper handleMDParams:[TATUserDataHelper mdParamsForMediaAPI]];
}

+ (NSString *)handleMDParams:(NSDictionary *)params {
    if (!params) {
        return nil;
    }
    NSData *jsonData = [params tat_modelToJSONData];
    NSString *base64String = [[jsonData gzippedData] base64EncodedString];
    base64String = [base64String stringByReplacingOccurrencesOfString:@"\n" withString:@""];
    base64String = [base64String stringByReplacingOccurrencesOfString:@"\r" withString:@""];
    // 进行urlencode
    base64String = [base64String stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
    return base64String;
}

+ (void)configBlockList:(NSArray *)blockList {
    if (![blockList containsObject:@"gps_longitude"] && ![blockList containsObject:@"gps_latitude"]) {
        [[TATClient current] getCurrentLocation];
    }
    NSDictionary *map = [TATUserDataHelper propertyKeyMap];
    NSMutableArray *propertyBlockList = [NSMutableArray array];
    for (NSString *key in map) {
        if ([blockList containsObject:key]) {
            [propertyBlockList addObject:map[key]];
        }
    }
    [TATClient current].blockList = propertyBlockList;
}

// 数据采集接口字段跟TATClient属性的映射关系
+ (NSDictionary *)propertyKeyMap {
    return @{@"app_key": @"appKey",
             @"package_name": @"bundleIdentifier",
             @"idfa": @"idfa",
             @"device_id": @"deviceId",
             @"gps_longitude": @"longitude",
             @"gps_latitude": @"latitude",
             @"os_type": @"systemName",
             @"net_type": @"networkType",
             @"sdk_version": @"sdkVersion",
             @"os_version": @"systemVersion",
             @"phone_brand": @"deviceModel",
             @"phone_model": @"deviceFullName",
             @"phone_electric": @"batteryPercent",
             @"phone_ip": @"ipv4",
             @"net_carrier": @"carrier",
             @"sensor_info": @"sensorListString",
             @"data_from": @"dataFromType",
             @"wifi_name": @"wifiName",
             @"wifi_list": @"wifilList",
             @"rom_capacity": @"totalMemory",
             @"available_rom": @"availableMemory",
             @"basestation_id": @"cellID",
             // V2.0 新增
             @"screen_resolution": @"screenResolution",
             @"screen_ppi": @"scale",
             @"cpu_core_number": @"cpuCount",
             @"imsi": @"imsi",
             @"device_name": @"deviceName",
             @"net_subtype": @"networkType",
             @"system_language": @"systemLanguage",
             @"app_version_name": @"appVersion",
             @"app_version_code": @"appBuild",
             @"app_path": @"appPath",
             @"app_name": @"appName",
             @"idfa_original": @"idfa_original",
    };
}

+ (NSDictionary *)mdParamsForCollectAPI {
    NSDictionary *params = @{@"time": @([NSDate tat_currentTimestamp]),
             @"app_key": [TATClient current].appKey ?: @"",
             @"package_name": [TATClient current].bundleIdentifier ?: @"",
             @"idfa": [TATClient current].idfa ?: @"",
             @"idfa_original": [TATClient current].idfa_original ?: @"",
             @"device_id": [TATClient current].deviceId ?: @"",
             @"gps_longitude": [TATClient current].longitude ?: @"",
             @"gps_latitude": [TATClient current].latitude ?: @"",
             @"os_type": [TATClient current].systemName ?: @"",
             @"net_type": [TATClient current].networkType ?: @"",
             @"sdk_version": [TATClient current].sdkVersion ?: @"",
             @"os_version": [TATClient current].systemVersion ?: @"",
             @"phone_brand": @"iPhone",
             @"phone_model": @"iPhone",
             @"phone_electric": @"",
             @"phone_ip": @"",
             @"net_carrier": @"",
             @"sensor_info": @"",
             @"data_from": [TATClient current].dataFromType ?: @"",
             @"wifi_name": @"",
             @"rom_capacity": @"",
             @"available_rom": @"",
             @"basestation_id": @"",
             // V2.0 新增
             @"screen_resolution": [TATClient current].screenResolution ?: @"",
             @"screen_ppi": [TATClient current].scale ?: @"",
             @"cpu_core_number": @"",
             @"cpu_frequency": @"",
             @"imsi": @"",
             @"device_name": @"",
             @"system_rom_name": @"",
             @"system_user_name": @"",
             @"net_subtype": [TATClient current].networkType ?: @"",
             @"system_language": @"",
             @"app_version_name": [TATClient current].appVersion ?: @"",
             @"app_version_code": [TATClient current].appBuild ?: @"",
             @"app_path": [TATClient current].appPath ?: @"",
             @"app_name": [TATClient current].appName ?: @"",
             @"tcid": [TATClient current].tcid ?: @"",
             @"is_empower": [TATClient current].authStatus ?: @"",
             @"system_file_time": @"",
             @"body_memory" : @"",
             @"simulate_idfa":  @"",
             @"phone_bootup_time" : @"",
             @"carrier_info": @"",
             @"develop_type": @"flutter"
    };
    //return [[self class] noEmptyParamsFrom:params];
    return params;
}

+ (NSDictionary *)mdParamsForMediaAPI {
    NSDictionary *params = @{@"idfa": [TATClient current].idfa ?: @"",
             @"device_id": [TATClient current].deviceId ?: @"",
             @"packageName": [TATClient current].bundleIdentifier ?: @"",
             @"api_version": @"1.0.0", // 版本号，写死1.0.0
             @"dataType": @"1", // 数据来源：1为SDK, 2为手动投放, 3为其他
             @"longitude": [TATClient current].longitude ?: @"",
             @"latitude": [TATClient current].latitude ?: @"",
             @"os": [TATClient current].systemName ?: @"",
             @"nt": [TATClient current].networkType ?: @"",
             @"sdk_version": [TATClient current].sdkVersion ?: @"",
             @"osversion": [TATClient current].systemVersion ?: @"",
             @"vendor": @"iPhone",
    };
    //return [[self class] noEmptyParamsFrom:params];
    return params;
}

+ (NSDictionary *)basicMdParamsForCrashAPI {
    NSDictionary *params = @{@"time": @([NSDate tat_currentTimestamp]),
             @"app_key": [TATClient current].appKey ?: @"",
             @"package_name": [TATClient current].bundleIdentifier ?: @"",
             @"idfa": [TATClient current].idfa ?: @"",
             @"device_id": [TATClient current].deviceId ?: @"",
             @"os_type": [TATClient current].systemName ?: @"",
             @"sdk_version": [TATClient current].sdkVersion ?: @"",
             @"os_version": @"",
             @"phone_brand": @"iPhone",
             @"phone_model": @"",
             @"data_from": [TATClient current].dataFromType ?: @"",
             @"tcid": [TATClient current].tcid ?: @"",
             @"is_empower": [TATClient current].authStatus ?: @""
    };
    return params;
}

// 去除没有实际值的键值对
+ (NSDictionary *)noEmptyParamsFrom:(NSDictionary *)dictionary {
    NSMutableDictionary *params = [NSMutableDictionary dictionary];
    for (NSString *key in dictionary) {
        if ([dictionary[key] isKindOfClass:[NSString class]]) {
            if([dictionary[key] tat_isNotEmpty]) {
                [params setObject:dictionary[key] forKey:key];
            }
        } else if (dictionary[key] != [NSNull null]) {
            [params setObject:dictionary[key] forKey:key];
        }
    }
    return params.copy;
}

@end
